@@ -37,6 +37,7 @@
 
 static EditLine* gEditLine;
 static HistoryW* gHistory;
+static sObject* gCmdLineStack;
 
 static void sig_int_editline(int signo)
 {
@@ -1828,6 +1829,9 @@ static void transform_lf_to_semicolon(wchar_t* buf, ALLOC wchar_t** result)
 #define OKCMD    -1    /* must be -1! */
 public int read_getcmd(EditLine *el, el_action_t *cmdnum, Char *ch);
 
+/* value to leave unused in line buffer */
+#define	EL_LEAVE	2
+
 public const Char * el_my_wgets(EditLine* el, int* nread, wchar_t* init_buffer, int cursor_pos)
 {
     int retval;
@@ -1840,6 +1844,13 @@ public const Char * el_my_wgets(EditLine* el, int* nread, wchar_t* init_buffer,
     c_macro_t *ma = &el->el_chared.c_macro;
 #endif /* FIONREAD */
 
+// --- this is hack by ab25cq -- //
+// clear buffer 
+    const int sz = (size_t)(gEditLine->el_line.limit - gEditLine->el_line.buffer + EL_LEAVE);
+    (void) memset(gEditLine->el_line.buffer, 0, sz);
+    gEditLine->el_line.lastchar = gEditLine->el_line.cursor = gEditLine->el_line.buffer;
+// -- end --
+
     if (nread == NULL)
         nread = &nrb;
     *nread = 0;
@@ -2238,7 +2249,21 @@ ALLOC char* editline(char* prompt, char* rprompt, char* text, int cursor_pos)
     do {
         gEditlineEditing = TRUE;
 
-        if(first) {
+        if(vector_count(gCmdLineStack) > 0) {
+            sObject* str = vector_pop_back(gCmdLineStack);
+
+            char* cstr = string_c_str(str);
+
+            const int len = strlen(cstr) + 1;
+            wchar_t* wstr = MALLOC(sizeof(wchar_t) * len);
+            mbstowcs(wstr, cstr, len);
+
+            line = el_my_wgets(gEditLine, &numc, wstr, -1);
+
+            FREE(wstr);
+            string_delete_on_malloc(str);
+        }
+        else if(first) {
             line = el_my_wgets(gEditLine, &numc, text2, cursor_pos);
             first = FALSE;
         }
@@ -2254,12 +2279,15 @@ ALLOC char* editline(char* prompt, char* rprompt, char* text, int cursor_pos)
         result = NULL;
     }
     else {
-        wchar_t* line2;
-        transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
-        line2[wcslen(line2)-1] = 0; // chomp
-        HistEventW ev;
-        history_w(gHistory, &ev, H_ENTER, line2);
-        FREE(line2);
+        char* ignore_history = getenv("HIST_IGNORE_SPACE");
+        if(ignore_history == NULL || strcmp(ignore_history, "on") != 0 || line[0] != ' ') {
+            wchar_t* line2;
+            transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
+            line2[wcslen(line2)-1] = 0; // chomp
+            HistEventW ev;
+            history_w(gHistory, &ev, H_ENTER, line2);
+            FREE(line2);
+        }
 
         const int size = MB_LEN_MAX * (wcslen(line) + 1);
         result = MALLOC(size);
@@ -2338,12 +2366,15 @@ ALLOC char* editline_user_castamized(char* prompt, char* rprompt, char* text, in
         result = NULL;
     }
     else {
-        wchar_t* line2;
-        transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
-        line2[wcslen(line2)-1] = 0; // chomp
-        HistEventW ev;
-        history_w(gHistory, &ev, H_ENTER, line2);
-        FREE(line2);
+        char* ignore_history = getenv("HIST_IGNORE_SPACE");
+        if(ignore_history == NULL || strcmp(ignore_history, "on") != 0 || line[0] != ' ') {
+            wchar_t* line2;
+            transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
+            line2[wcslen(line2)-1] = 0; // chomp
+            HistEventW ev;
+            history_w(gHistory, &ev, H_ENTER, line2);
+            FREE(line2);
+        }
 
         const int size = MB_LEN_MAX * (wcslen(line) + 1);
         result = MALLOC(size);
@@ -2420,12 +2451,15 @@ ALLOC char* editline_no_completion(char* prompt, char* rprompt, char* text, int
         result = NULL;
     }
     else {
-        wchar_t* line2;
-        transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
-        line2[wcslen(line2)-1] = 0; // chomp
-        HistEventW ev;
-        history_w(gHistory, &ev, H_ENTER, line2);
-        FREE(line2);
+        char* ignore_history = getenv("HIST_IGNORE_SPACE");
+        if(ignore_history == NULL || strcmp(ignore_history, "on") != 0 || line[0] != ' ') {
+            wchar_t* line2;
+            transform_lf_to_semicolon((wchar_t*)line, ALLOC &line2);
+            line2[wcslen(line2)-1] = 0; // chomp
+            HistEventW ev;
+            history_w(gHistory, &ev, H_ENTER, line2);
+            FREE(line2);
+        }
 
         const int size = MB_LEN_MAX * (wcslen(line) + 1);
         result = MALLOC(size);
@@ -2478,13 +2512,13 @@ BOOL cmd_editline_history(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         wcstombs(buf, (wchar_t*)ev.str, size);
 
         if(!fd_write(nextout, buf, strlen(buf))) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             FREE(buf);
             return FALSE;
         }
         if(!fd_writec(nextout, '\n')) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             FREE(buf);
             return FALSE;
@@ -2601,13 +2635,13 @@ BOOL cmd_editline_line_buffer(sObject* nextin, sObject* nextout, sRunInfo* runin
         wcstombs(mbs, li->buffer, size);
 
         if(!fd_write(nextout, mbs, strlen(mbs))) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             FREE(mbs);
             return FALSE;
         }
         if(!fd_write(nextout, "\n", 1)) {
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             FREE(mbs);
             return FALSE;
@@ -2626,6 +2660,7 @@ BOOL cmd_editline_line_buffer(sObject* nextin, sObject* nextout, sRunInfo* runin
 static el_action_t ed_my_newline(EditLine *el, int ch)
 {
     el_action_t res = 0;
+
     stack_start_stack();
 
     sObject* block = BLOCK_NEW_STACK();
@@ -2664,6 +2699,25 @@ static el_action_t ed_my_newline(EditLine *el, int ch)
     return res;
 }
 
+static el_action_t ed_commandline_stack(EditLine *el, int ch)
+{
+    el_action_t res = 0;
+
+    const LineInfoW *lf = el_wline(el);
+    const int size = MB_LEN_MAX * (wcslen(lf->buffer) + 1);
+    char* buf = MALLOC(size);
+    wcstombs(buf, lf->buffer, size);
+
+    vector_add(gCmdLineStack, STRING_NEW_MALLOC(buf));
+
+    FREE(buf);
+
+    ch_reset(el, 0);
+    re_refresh(gEditLine);
+
+    return res;
+}
+
 static void editline_additional_keybind_vi()
 {
     el_wset(gEditLine, EL_ADDFN, L"ed-complete", L"Complete argument", complete);
@@ -2674,6 +2728,7 @@ static void editline_additional_keybind_vi()
     el_wset(gEditLine, EL_ADDFN, L"ed-my-newline", L"Pressing Enter", ed_my_newline);
     el_wset(gEditLine, EL_BIND, L"^J", L"ed-my-newline", NULL);
     el_wset(gEditLine, EL_BIND, L"^M", L"ed-my-newline", NULL);
+    el_wset(gEditLine, EL_BIND, L"^[q", L"ed-commandline-stack", NULL);
 }
 
 static void editline_additional_keybind_emacs()
@@ -2686,6 +2741,9 @@ static void editline_additional_keybind_emacs()
     el_wset(gEditLine, EL_ADDFN, L"ed-my-newline", L"Pressing Enter", ed_my_newline);
     el_wset(gEditLine, EL_BIND, L"^J", L"ed-my-newline", NULL);
     el_wset(gEditLine, EL_BIND, L"^M", L"ed-my-newline", NULL);
+    el_wset(gEditLine, EL_ADDFN, L"ed-commandline-stack", L"Commandline stack", ed_commandline_stack);
+    el_wset(gEditLine, EL_BIND, L"^[q", L"ed-commandline-stack", NULL);
+//el_wset(gEditLine, EL_BIND, NULL);
 }
 
 BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
@@ -2707,7 +2765,7 @@ BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     else if(runinfo->mArgsNumRuntime == 1) {
         char* home = getenv("HOME");
         if(home == NULL) {
-            err_msg("$HOME is NULL", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("$HOME is NULL", runinfo->mSName, runinfo->mSLine);
             return FALSE;
         }
 
@@ -2715,7 +2773,7 @@ BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         snprintf(path, PATH_MAX, "%s/.xyzsh/tmpfile", home);
         FILE* file = fopen(path, "w");
         if(file == NULL) {
-            err_msg("can't ceate tmpfile", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("can't ceate tmpfile", runinfo->mSName, runinfo->mSLine);
             return FALSE;
         }
 
@@ -2734,7 +2792,7 @@ BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             int size = read(fd, buf, BUFSIZ);
 
             if(size < 0) {
-                err_msg("can't read temporaly file", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                err_msg("can't read temporaly file", runinfo->mSName, runinfo->mSLine);
                 close(fd);
                 return FALSE;
             }
@@ -2743,7 +2801,7 @@ BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
             }
             else {
                 if(!fd_write(nextout, buf, size)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     fclose(file);
                     return FALSE;
@@ -2768,7 +2826,7 @@ BOOL cmd_editline_keybind(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         mbstowcs(wcsfunc, func, len2);
         
         if(el_wset(gEditLine, EL_BIND, wcskey, wcsfunc, NULL) < 0) {
-            err_msg("error on bind function to a key", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg("error on bind function to a key", runinfo->mSName, runinfo->mSLine);
 
             FREE(wcskey);
             FREE(wcsfunc);
@@ -2805,7 +2863,7 @@ BOOL cmd_editline_point(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
         int n = snprintf(buf, BUFSIZ, "%d\n", cursor_pos);
         if(!fd_write(nextout, buf, n)) {
             sCommand* command = runinfo->mCommand;
-            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -2904,7 +2962,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                         sObject* block = SCOMPLETION(compl).mBlock;
                         if(!fd_write(nextout, SBLOCK(block).mSource, strlen(SBLOCK(block).mSource))) 
                         {
-                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                            err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine);
                             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                             return FALSE;
                         }
@@ -2916,7 +2974,7 @@ BOOL cmd_completion(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
                     }
                 }
                 else {
-                    err_msg("There is no object", runinfo->mSName, runinfo->mSLine, runinfo->mArgsRuntime[i]);
+                    err_msg("There is no object", runinfo->mSName, runinfo->mSLine);
 
                     return FALSE;
                 }
@@ -2979,7 +3037,7 @@ BOOL run_completion(sObject* compl, sObject* nextin, sObject* nextout, sRunInfo*
         else if(rcode == RCODE_EXIT) {
         }
         else {
-            err_msg_adding("run time error\n", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+            err_msg_adding("run time error\n", runinfo->mSName, runinfo->mSLine);
         }
 
         (void)vector_pop_back(gStackFrames);
@@ -3076,10 +3134,17 @@ void xyzsh_editline_init()
     el_source(gEditLine, NULL);
 
     editline_additional_keybind_emacs();
+    gCmdLineStack = VECTOR_NEW_MALLOC(10);
 }
 
 void xyzsh_editline_final()
 {
+    int i;
+    for(i=0; i<vector_count(gCmdLineStack); i++) {
+        string_delete_on_malloc(vector_item(gCmdLineStack, i));
+    }
+    vector_delete_on_malloc(gCmdLineStack);
+
     //if(gHistorySuggestionTail) FREE(gHistorySuggestionTail);
 
     /// write history ///
