@@ -53,7 +53,7 @@ BOOL add_object_to_objects_in_pipe(sObject* object, sRunInfo* runinfo, sCommand*
 
 /*
     if(vector_count(gObjectsInPipe) > XYZSH_OBJECTS_IN_PIPE_MAX) {
-        err_msg("overflow objects in pipe", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        err_msg("overflow objects in pipe", runinfo->mSName, runinfo->mSLine);
         return FALSE;
     }
 */
@@ -149,7 +149,7 @@ BOOL forground_job(int num)
         return TRUE;
     }
 
-    err_msg("invalid job number", "fg", 1, "fg");
+    err_msg("invalid job number", "fg", 1);
     return FALSE;
 }
 
@@ -249,7 +249,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo,
     char buf[BUFSIZ+1];
     while(1) {
         if(gXyzshSigInt) {
-            err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine, program);
+            err_msg("signal interrupt1", runinfo->mSName, runinfo->mSLine);
             gXyzshSigInt = FALSE;
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
@@ -269,7 +269,7 @@ static BOOL nextout_reader(sObject* nextout, int* pipeoutfds, sRunInfo* runinfo,
         }
 
         if(!fd_write(nextout, buf, size)) {
-            err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine, program);
+            err_msg("signal interrupt3", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -342,7 +342,7 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
                     kill(nextin_reader_pid, SIGCONT);
                 }
                 else if(WIFSIGNALED(status)) {
-                    err_msg("signal interrupt4", runinfo->mSName, runinfo->mSLine, program);
+                    err_msg("signal interrupt4", runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
                 else {
@@ -356,7 +356,7 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
 
             if(WIFSIGNALED(status) || WIFSTOPPED(status))
             {
-                err_msg("signal interrupt5", runinfo->mSName, runinfo->mSLine, program);
+                err_msg("signal interrupt5", runinfo->mSName, runinfo->mSLine);
                 return FALSE;
             }
         }
@@ -374,7 +374,7 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
                     kill(pid, SIGCONT);
                 }
                 else if(WIFSIGNALED(status)) {
-                    err_msg("signal interrupt6", runinfo->mSName, runinfo->mSLine, program);
+                    err_msg("signal interrupt6", runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
                 else {
@@ -387,7 +387,7 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
             pid = waitpid(pid, &status, WUNTRACED);
 
             if(WIFSTOPPED(status)) {
-                err_msg("signal interrupt7", runinfo->mSName, runinfo->mSLine, program);
+                err_msg("signal interrupt7", runinfo->mSName, runinfo->mSLine);
 
                 kill(pid, SIGKILL);
                 pid = waitpid(pid, &status, WUNTRACED);
@@ -412,12 +412,12 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
 
                     char buf[BUFSIZ];
                     snprintf(buf, BUFSIZ, "command not found");
-                    err_msg(buf, runinfo->mSName, runinfo->mSLine, program);
+                    err_msg(buf, runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
             }
             else if(WIFSIGNALED(status)) {  // a xyzsh external program which is not a last command can't be stopped by CTRL-Z
-                err_msg("signal interrupt8", runinfo->mSName, runinfo->mSLine, program);
+                err_msg("signal interrupt8", runinfo->mSName, runinfo->mSLine);
 
                 if(tcsetpgrp(0, getpgid(0)) < 0) {
                     perror("tcsetpgrp(xyzsh)");
@@ -486,7 +486,7 @@ static BOOL wait_child_program(pid_t pid, pid_t nextin_reader_pid, int nextout2,
 
                         char buf[BUFSIZ];
                         snprintf(buf, BUFSIZ, "command not found");
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine, program);
+                        err_msg(buf, runinfo->mSName, runinfo->mSLine);
                         return FALSE;
                     }
                 }
@@ -795,7 +795,7 @@ BOOL run_function(sObject* fun, sObject* nextin, sObject* nextout, sRunInfo* run
         }
         else {
             char buf[BUFSIZ];
-            err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine);
 
             runinfo->mRCode = rcode;
 
@@ -825,7 +825,7 @@ static BOOL run_block(sObject* block, sObject* nextin, sObject* nextout, sRunInf
     int rcode = 0;
     if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, runinfo->mRunningObject)) {
         sCommand* command = runinfo->mCommand;
-        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine);
         runinfo->mRCode = rcode;
         return FALSE;
     }
@@ -869,7 +869,7 @@ static BOOL run_alias(sObject* alias, sObject* nextin, sObject* nextout, sRunInf
     int rcode = 0;
     if(!run(block, nextin, nextout, &rcode, runinfo->mCurrentObject, alias)) {
         sCommand* command = runinfo->mCommand;
-        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+        err_msg_adding("run time error", runinfo->mSName, runinfo->mSLine);
 
         (void)vector_pop_back(gStackFrames);
         SALIAS(alias).mLocalObjects = local_objects;
@@ -906,7 +906,7 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
             }
             if(runinfo->mRCode == RCODE_NFUN_INVALID_USSING) {
                 sCommand* command = runinfo->mCommand;
-                err_msg("invalid command using", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
+                err_msg("invalid command using", runinfo->mSName, runinfo->mSLine);
                 return FALSE;
             }
             break;
@@ -951,23 +951,13 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
             int size = strlen(str);
             if(!fd_write(nextout, str, size)) {
                 sCommand* command = runinfo->mCommand;
-                if(command) {
-                    err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                }
-                else {
-                    err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, "");
-                }
+                err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
             if(!fd_write(nextout, "\n", 1)) {
                 sCommand* command = runinfo->mCommand;
-                if(command) {
-                    err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                }
-                else {
-                    err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, "");
-                }
+                err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -1009,23 +999,13 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
                 char* key = hash_loop_key(it);
                 if(!fd_write(nextout, key, strlen(key))) {
                     sCommand* command = runinfo->mCommand;
-                    if(command) {
-                        err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                    }
-                    else {
-                        err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine, "");
-                    }
+                    err_msg("signal interrupt9", runinfo->mSName, runinfo->mSLine);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
                 if(!fd_write(nextout, "\n", 1)) {
                     sCommand* command = runinfo->mCommand;
-                    if(command) {
-                        err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                    }
-                    else {
-                        err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine, "");
-                    }
+                    err_msg("signal interrupt10", runinfo->mSName, runinfo->mSLine);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
@@ -1064,12 +1044,7 @@ BOOL run_object(sObject* object, sObject* nextin, sObject* nextout, sRunInfo* ru
         default: {
             char buf[BUFSIZ];
             sCommand* command = runinfo->mCommand;
-            if(command) {
-                err_msg("can't run this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            }
-            else {
-                err_msg("can't run this object", runinfo->mSName, runinfo->mSLine, "");
-            }
+            err_msg("can't run this object", runinfo->mSName, runinfo->mSLine);
             }
             return FALSE;
     }
@@ -1116,14 +1091,14 @@ static BOOL ready_for_redirect(sObject** nextout, sObject** nextin, int* opened_
                 if(fd < 0) {
                     char buf[BUFSIZ];
                     snprintf(buf, BUFSIZ, "redirect: error with openning %s file", runinfo->mRedirectsFileNamesRuntime[i]);
-                    err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
+                    err_msg(buf, runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
 
                 if(!bufsiz_read(fd, ALLOC &result, &result_len, &result_size)) {
                     char buf[BUFSIZ];
                     snprintf(buf, BUFSIZ, "redirect: error with reading %s file", runinfo->mRedirectsFileNamesRuntime[i]);
-                    err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
+                    err_msg(buf, runinfo->mSName, runinfo->mSLine);
                     return FALSE;
                 }
                 (void)close(fd);
@@ -1138,7 +1113,7 @@ static BOOL ready_for_redirect(sObject** nextout, sObject** nextin, int* opened_
                     if(fd < 0) {
                         char buf[BUFSIZ];
                         snprintf(buf, BUFSIZ, "redirect: error with openning %s file", runinfo->mRedirectsFileNamesRuntime[i]);
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
+                        err_msg(buf, runinfo->mSName, runinfo->mSLine);
                         return FALSE;
                     }
                     *nextout = FD2_NEW_STACK(fd);
@@ -1153,7 +1128,7 @@ static BOOL ready_for_redirect(sObject** nextout, sObject** nextin, int* opened_
                     if(fd < 0) {
                         char buf[BUFSIZ];
                         snprintf(buf, BUFSIZ, "redirect: error with openning %s file", runinfo->mRedirectsFileNamesRuntime[i]);
-                        err_msg(buf, runinfo->mSName, runinfo->mSLine, "");
+                        err_msg(buf, runinfo->mSName, runinfo->mSLine);
                         return FALSE;
                     }
                     *nextout = FD2_NEW_STACK(fd);
@@ -1229,7 +1204,7 @@ static BOOL inherit_method(sRunInfo* runinfo, sObject** object, sCommand* comman
 
         case T_NFUN:
         case T_BLOCK:
-            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            err_msg("There is not a parent object", runinfo->mSName, runinfo->mSLine);
             return FALSE;
 
         default:
@@ -1262,7 +1237,7 @@ BOOL get_object_from_command_name(sCommand* command, sObject* current_object, sO
                 *object = uobject_item(*object, command->mMessages[i++]);
             }
             else {
-                err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine);
                 return FALSE;
             }
         }
@@ -1272,7 +1247,7 @@ BOOL get_object_from_command_name(sCommand* command, sObject* current_object, sO
             *object = uobject_item(*object, command->mArgs[0]);
         }
         else {
-            err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            err_msg("invalid message passing", runinfo->mSName, runinfo->mSLine);
             return FALSE;
         }
     }
@@ -1300,12 +1275,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         if(pipein == gStdin && SFD(gStdin).mBufLen == 0 && !isatty(0)) {
             if(!stdin_read_out()) {
                 sCommand* command = runinfo->mCommand;
-                if(command) {
-                    err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                }
-                else {
-                    err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine, "");
-                }
+                err_msg("signal interrupt11", runinfo->mSName, runinfo->mSLine);
                 runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                 return FALSE;
             }
@@ -1334,12 +1304,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                 char* line = vector_item(SFD(pipein).mLines, i);
                 if(!fd_write(nextin, line, strlen(line))) {
                     sCommand* command = runinfo->mCommand;
-                    if(command) {
-                        err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-                    }
-                    else {
-                        err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, "");
-                    }
+                    err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine);
                     runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
                     return FALSE;
                 }
@@ -1351,12 +1316,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         nextin = FD_NEW_STACK();
         if(!fd_write(nextin, SFD(gGlobalPipe).mBuf, SFD(gGlobalPipe).mBufLen)) {
             sCommand* command = runinfo->mCommand;
-            if(command) {
-                err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
-            }
-            else {
-                err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine, "");
-            }
+            err_msg("signal interrupt12", runinfo->mSName, runinfo->mSLine);
             runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
             return FALSE;
         }
@@ -1421,7 +1381,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
                     }
                 }
                 else {
-                    err_msg("there is not this object", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    err_msg("there is not this object", runinfo->mSName, runinfo->mSLine);
                     sRunInfo_command_delete(runinfo);
                     return FALSE;
                 }
@@ -1462,10 +1422,10 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
     if(nest_level == 50) {
         if(SBLOCK(block).mStatmentsNum > 0) {
             sStatment* statment = SBLOCK(block).mStatments + 0;
-            err_msg("stack over flow", statment->mFName, statment->mLine, "system");
+            err_msg("stack over flow", statment->mFName, statment->mLine);
         }
         else {
-            err_msg("stack over flow", "xyzsh", 0, "system");
+            err_msg("stack over flow", "xyzsh", 0);
         }
         nest_level--;
 
@@ -1491,24 +1451,48 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         runinfo.mStatment = statment;
         runinfo.mNestLevel = nest_level;
 
-        if(!statment_tree(statment, pipein, pipeout, &runinfo, current_object)) {
-            fd_clear(gStdout);
-            fd_clear(gStderr);
+        if(statment->mFlags & STATMENT_FLAGS_KIND_NODETREE) {
+            runinfo.mCurrentObject = current_object;
+            if(!node_tree(statment, pipein, pipeout, &runinfo, current_object)) {
+                fd_clear(gStdout);
+                fd_clear(gStderr);
 
-            *rcode = runinfo.mRCode;
+                *rcode = runinfo.mRCode;
 
-            stack_end_stack();
-            nest_level--;
+                stack_end_stack();
+                nest_level--;
 
-            if(nest_level == 0) {
-                vector_clear(gObjectsInPipe);
+                if(nest_level == 0) {
+                    vector_clear(gObjectsInPipe);
+                }
+
+                if(!(n++ % 32)) (void)gc_sweep();
+
+                (void)vector_pop_back(gRunningObjects);
+                (void)vector_pop_back(gRunningObjects);
+                return FALSE;
             }
+        }
+        else {
+            if(!statment_tree(statment, pipein, pipeout, &runinfo, current_object)) {
+                fd_clear(gStdout);
+                fd_clear(gStderr);
 
-            if(!(n++ % 32)) (void)gc_sweep();
+                *rcode = runinfo.mRCode;
 
-            (void)vector_pop_back(gRunningObjects);
-            (void)vector_pop_back(gRunningObjects);
-            return FALSE;
+                stack_end_stack();
+                nest_level--;
+
+                if(nest_level == 0) {
+                    vector_clear(gObjectsInPipe);
+                }
+
+                if(!(n++ % 32)) (void)gc_sweep();
+
+                (void)vector_pop_back(gRunningObjects);
+                (void)vector_pop_back(gRunningObjects);
+                return FALSE;
+            }
         }
 
         /// flash stdout and stderr ///
@@ -1518,7 +1502,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
                 fd_clear(gStderr);
 
                 sCommand* command = runinfo.mCommand;
-                err_msg("signal interrupt13", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
+                err_msg("signal interrupt13", runinfo.mSName, runinfo.mSLine);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1538,7 +1522,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
                 fd_clear(gStderr);
 
                 sCommand* command = runinfo.mCommand;
-                err_msg("signal interrupt14", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
+                err_msg("signal interrupt14", runinfo.mSName, runinfo.mSLine);
                 *rcode = RCODE_SIGNAL_INTERRUPT;
                 stack_end_stack();
                 nest_level--;
@@ -1556,7 +1540,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         if(gXyzshSigUser) {
             gXyzshSigUser = FALSE;
             sCommand* command = runinfo.mCommand;
-            err_msg("command not found", runinfo.mSName, runinfo.mSLine, ""); //command->mArgs[0]);
+            err_msg("command not found", runinfo.mSName, runinfo.mSLine);
             stack_end_stack();
             nest_level--;
             fd_clear(gStdout);
@@ -1572,7 +1556,7 @@ BOOL run(sObject* block, sObject* pipein, sObject* pipeout, int* rcode, sObject*
         else if(runinfo.mRCode == 128+SIGINT || gXyzshSigInt) {
             gXyzshSigInt = FALSE;
             sCommand* command = runinfo.mCommand;
-            err_msg("signal interrupt15", runinfo.mSName, runinfo.mSLine, command->mArgs[0]);
+            err_msg("signal interrupt15", runinfo.mSName, runinfo.mSLine);
             *rcode = RCODE_SIGNAL_INTERRUPT;
             stack_end_stack();
             nest_level--;
