@@ -1,6 +1,8 @@
 #ifndef __TERMEMU_H
 #define __TERMEMU_H
 
+#include "config.h"
+
 #if defined(HAVE_CURSES_H)
 #include <curses.h>
 #elif defined(HAVE_NCURSES_H)
@@ -62,5 +64,7 @@ extern BOOL temulator_write(sTEmulator* self, int key);
 BOOL temulator_draw_on_curses(sTEmulator* self, WINDOW* win, int win_pos_y, int win_pos_x);
 BOOL temulator_resize(sTEmulator* self, int height, int width);
 
+void run_on_temulator(fTEmulator fun, void* arg);
+
 #endif
 
