@@ -218,9 +218,12 @@ typedef struct {
 #define STATMENT_FLAGS_OROR 0x400000
 #define STATMENT_FLAGS_ANDAND 0x800000
 #define STATMENT_FLAGS_KIND_NODETREE 0x1000000
+#define STATMENT_FLAGS_NODETREE_OUTPUT 0x2000000
+#define STATMENT_FLAGS_NODETREE_PASTE 0x4000000
 #define STATMENT_FLAGS_CONTEXT_PIPE_NUMBER 0xFFFF
 
 #define STATMENT_COMMANDS_MAX 12
+#define STATMENT_NODE_MAX 12
 #define XYZSH_OPTION_MAX 16
 
 struct _sNodeTree;
@@ -234,12 +237,15 @@ typedef struct {
         struct {
             sCommand mCommands[STATMENT_COMMANDS_MAX];
             unsigned char mCommandsNum;
-
-            char* mFName;
-            ushort mLine;
         };
-        struct _sNodeTree* mNodeTree;
+        struct {
+            struct _sNodeTree* mNodeTree[STATMENT_NODE_MAX];
+            unsigned char mNodeTreeNum;
+        };
     };
+
+    char* mFName;
+    ushort mLine;
 } sStatment;
 
 #define COMPLETION_FLAGS_BLOCK_OR_ENV_NUM 0xff
@@ -370,7 +376,8 @@ typedef struct {
 #define T_COMPLETION 14
 #define T_EXTOBJ 15
 #define T_ALIAS 16
-#define T_TYPE_MAX 17
+#define T_INT 17
+#define T_TYPE_MAX 18
 
 typedef struct _sObject {
     int mFlags;         // contains a kind of the above and a mark flag, user object flag
@@ -394,6 +401,7 @@ typedef struct _sObject {
         external_prog_obj uExternalProg;
         external_obj uExternalObj;
         alias_obj uAlias;
+        int uInt;
     };
 } sObject;
 
@@ -413,6 +421,7 @@ typedef struct _sObject {
 #define SEXTOBJ(o) (o)->uExternalObj
 #define SCOMPLETION(o) (o)->uCompletion
 #define SALIAS(o) (o)->uAlias
+#define SINT(o) (o)->uInt
 
 void gc_init(int pool_size);
 void gc_final();
@@ -801,6 +810,9 @@ sObject* alias_new_on_gc(sObject* block, BOOL user_object, sObject* parent);
 #define ALIAS_NEW_GC(o, o2, o3) alias_new_on_gc(o, o2, o3);
 int alias_gc_children_mark(sObject* self);
 
+sObject* int_new_on_stack(int value);
+#define INT_NEW_STACK(o) int_new_on_stack(o);
+
 /// structure used from run function, which contains run time info
 typedef struct _sRunInfo {
     char* mSName;
@@ -1074,26 +1086,36 @@ BOOL get_object_prefix_and_name_from_argument(sObject** object, sObject* name, c
 BOOL get_object_from_command_name(sCommand* command, sObject* current_object, sObject** object, sObject** reciever, sRunInfo* runinfo);
 
 enum eOperand { 
-    kOpAdd, kOpSub, kOpMult, kOpDiv
+    kOpAdd, kOpSub, kOpMult, kOpDiv, kOpEqual, kOpEqualPlus, kOpEqualMinus, kOpEqualMult, kOpEqualDiv, kOpEqualMod, kOpEqualShiftLeft, kOpEqualShiftRight, kOpEqualAnd, kOpEqualOr, kOpEqualXor, kOpAndAnd, kOpOrOr, kOpEqEq, kOpNotEqEq, kOpGt, kOpGte, kOpLt, kOpLte, kOpQuestion, kOpOr, kOpXor, kOpAnd, kOpLeftShift, kOpRightShift, kOpMod, kOpPlusPlus, kOpMinusMinus, kOpTilda, kOpExclamation, kOpPlusPlus2, kOpMinusMinus2, kOpStrAdd, kOpStrEqEq, kOpStrNotEqEq, kOpStrCmp, kOpStrLt, kOpStrGt, kOpStrLe, kOpStrGe
 };
 
 struct _sNodeTree {
-    unsigned char mType;   // 0: operand 1: value
+    unsigned char mType;   // 0: operand 1: value 2: local variable 3: global variable 4:string value 5:function name
 
     union {
         enum eOperand mOperand;
         int mValue;
+        sObject* mVarName;
+        sObject* mStringValue;
+        struct {
+            char* mFunName;
+            char* mFunArg;
+        };
     };
 
     struct _sNodeTree* mLeft;
     struct _sNodeTree* mRight;
+    struct _sNodeTree* mMiddle;
 };
 
-sNodeTree* sNodeTree_create_operand(enum eOperand operand, sNodeTree* left, sNodeTree* right);
-sNodeTree* sNodeTree_create_value(int value, sNodeTree* left, sNodeTree* right);
+sNodeTree* sNodeTree_create_operand(enum eOperand operand, sNodeTree* left, sNodeTree* right, sNodeTree* middle);
+sNodeTree* sNodeTree_create_value(int value, sNodeTree* left, sNodeTree* right, sNodeTree* middle);
+sNodeTree* sNodeTree_create_var(char* var_name, sNodeTree* left, sNodeTree* right, sNodeTree* middle);
+sNodeTree* sNodeTree_clone(sNodeTree* source);
 void sNodeTree_free(sNodeTree* self);
+BOOL node_tree(sStatment* statment, sObject* nextin, sObject* nextout, sRunInfo* runinfo, sObject* current_object);
 
-BOOL expression_add_sub(sNodeTree** node, char** p, char* sname, int* sline);
+BOOL node_expression(ALLOC sNodeTree** node, char** p, char* sname, int* sline);
 
 //////////////////////////////////////////////////////////////////////
 // xyzsh API
@@ -1110,8 +1132,8 @@ int xyzsh_job_num();
 BOOL xyzsh_run(int* rcode, sObject* block, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object);
 BOOL xyzsh_eval(int* rcode, char* cmd, char* source_name, fXyzshJobDone xyzsh_job_done_, sObject* nextin, sObject* nextout, int argc, char** argv, sObject* current_object);
 
-void err_msg(char* msg, char* sname, int line, char* command);
-void err_msg_adding(char* msg, char* sname, int line, char* command);
+void err_msg(char* msg, char* sname, int line);
+void err_msg_adding(char* msg, char* sname, int line);
 
 sObject* access_object(char* name, sObject** current, sObject* running_object);
 sObject* access_object2(char* name, sObject* current, sObject* running_object);
